#!/usr/bin/env python

"""
Examples of the use of the EaterWorld class
"""

import random
from typing import Iterable
import sys

from wumpus import InformedPlayer
from wumpus.gridworld import Eater, EaterWorld, Food


class MyPlayer(InformedPlayer):
    """
    Informed player demonstrating the use of the start episode method to inspect the world.
    """

    def _say(self, text: str):
        print(self.name + ' says: ' + text)

    def start_episode(self, world: EaterWorld):
        """
        Print the description of the world before starting.
        """
        # keep track of the reward
        self.reward = 0

        self._say('Episode starting for player {}'.format(self.name))

        # inspect the objects in the world
        food_locations = []
        eater_location = None
        for o in world.objects:
            if isinstance(o, Eater):
                eater_location = (o.location.x, o.location.y)
            elif isinstance(o, Food):
                food_locations.append((o.location.x, o.location.y))

        # get the list of blocks
        block_locations = sorted((bl.x, bl.y) for bl in world.blocks)

        # Print the description of the world
        self._say('World size: {}x{}'.format(world.size.x, world.size.y))
        self._say('Eater agent in {}'.format(eater_location))
        self._say('Food in {}'.format(sorted(food_locations)))
        self._say('Blocks in {}'.format(block_locations))

        self._say('Available actions: {}'.format({a.name: a.value for a in Eater.Actions}))

    def end_episode(self):
        """Method called at the when an episode is completed."""
        self._say('Episode completed, my reward is {}'.format(self.reward))

    # random player
    def play(self, turn: int, percept: Eater.Percept, actions: Iterable[Eater.Actions]) -> Eater.Actions:
        actions_lst = list(actions)
        next_move = actions_lst[random.randint(0, len(actions) - 1)]
        self._say('I see {}, my next move is {}'.format(percept, next_move.name))
        return next_move

    def feedback(self, action: Eater.Actions, reward: int, percept: Eater.Percept):
        """Receive in input the reward of the last action and the resulting state. The function is called right after the execution of the action."""
        self._say('Moved to {} with reward {}'.format(percept.position, reward))
        self.reward += reward


MAP_STR = """
################
#    #    #    #
#    #    #    #
#    #         #
#    #    #    #
###           ##
"""


def main(*args):
    """
    Play a random EaterWorld episode using the default player
    """

    player_class = MyPlayer

    world = EaterWorld.random(MAP_STR)
    player = player_class('Hungry.Monkey')

    world.run_episode(player, horizon=20)

    return 0


if __name__ == "__main__":
    sys.exit(main(*sys.argv[1:]))
